// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden        
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/* -*-antlr-*- */
header {

    package de.uka.ilkd.key.parser;

    import de.uka.ilkd.key.logic.*;
    import de.uka.ilkd.key.logic.op.*;
    import de.uka.ilkd.key.java.JavaInfo;
    import de.uka.ilkd.key.java.abstraction.KeYJavaType;
    import de.uka.ilkd.key.java.*;
    import de.uka.ilkd.key.java.reference.*;
    import java.util.HashSet;

}

/** 
 * A parser for modifies clauses,
 * i.e. FieldReferences.
 */  

class ModifiesParser extends Parser;
options {
	importVocab=KeYLexer;
	k = 3;
	defaultErrorHandler=false;
}

{
	private Namespace ns;
	private HashSet fields = new HashSet();
	private JavaInfo javaInfo;

	public ModifiesParser(TokenStream lexer,
			  Namespace ns,
			  JavaInfo javaInfo) {
	    this(lexer);
	    this.ns = ns;
	    this.javaInfo = javaInfo;
	}
		
	public Namespace namespace() {
	    return ns;
        }

	public JavaInfo getJavaInfo() {
	    return javaInfo;
	}

	public HashSet modifiesElements() {
	    return fields;
	}

}



top {ProgramElement f;} : f = fieldref {fields.add(f);} 
			   (COMMA f = fieldref {fields.add(f);})* ;

fieldref returns [ProgramElement pe = null]  {String y = null;
					      KeYJavaType last = null;
					     } :
		(
			(y = programvar)
			{ pe = (ProgramVariable) ns.lookup(new Name(y));
			  if (pe == null) {
		            throw new 
			    RuntimeException("Program Variable not found: "+y);
       			  } 
			  last = ((ProgramVariable)pe).getKeYJavaType();
			}
			(DOT y = programvar
			{
			  ProgramVariable x = javaInfo.getAttribute(y, last);
			  if (x == null) {
			    throw new
			    RuntimeException("Attribute "+y+" not found in "
						+last+"   "+pe);
			  }
			  pe = new FieldReference(x, (ReferencePrefix) pe);
			  last = ((ProgramVariable)x).getKeYJavaType();
			})*
		) ;

programvar returns [String pv = null]:
   id:IDENT 
	{pv = (String) id.getText(); 
	} ;
