package de.uka.ilkd.key.proof.mgt;

import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.proof.init.*;
import de.uka.ilkd.key.gui.KeYMediator;
import java.util.*;
import org.apache.log4j.Logger;
import javax.swing.JOptionPane;

public class GlobalProofMgt {

    private static final GlobalProofMgt INSTANCE = new GlobalProofMgt();

    private Map envKeyToEnv = new HashMap();

    private KeYMediator mediator;

    private Logger mgtLogger = Logger.getLogger("key.proof.mgt");

    public static GlobalProofMgt getInstance() {
	return INSTANCE;
    }

    public void setMediator(KeYMediator m) {
	mediator = m;
    }

    public ProofEnvironment getProofEnvironment(JavaModel jmodel, 
						RuleConfig ruleConfig) {
	if (jmodel==null || jmodel==JavaModel.NO_MODEL) {
	    return null;
	}
	ProofEnvironment env 
	    = (ProofEnvironment) envKeyToEnv.get(new EnvKey(jmodel, ruleConfig));
	return env;
    }

    public void registerProofEnvironment(ProofEnvironment env) {
	envKeyToEnv.put(new EnvKey(env.getJavaModel(), env.getRuleConfig()), 
			env);
    }

    public void tryReuse(ProofList plist) {
	for (int i=0; i<plist.size(); i++) {
	    tryReuse(plist.get(i));
	}
    }

    public void tryReuse(Proof proof) {
        Proof[] prevs = lookupPrevious(proof);
        if (prevs.length>0) {
	    String[] prevNames = new String[prevs.length];
	    for (int i=0; i<prevNames.length; i++) {
		prevNames[i]="From "+prevs[i].env().description();
	    }
	    String pname = (String) JOptionPane.showInputDialog
		(mediator.mainFrame(), "Found previous proofs for this PO in "
		 +proof.env().description()+"\n"
		 +"Mark up for re-use? (previous marks will be lost)",
		 "Re-Use", JOptionPane.QUESTION_MESSAGE, null, prevNames, null);
	    
	    if (pname!=null) {
		int i=0;
		while (!pname.equals(prevNames[i])) i++;
		tryReuse(proof, prevs[i]);
	    }
        }
    }

    public void tryReuse(Proof proof, Proof prev) {
        String error = null;	
	CvsRunner cvs = new CvsRunner();
	String diff;
	Node.clearReuseCandidates();
	ReuseFrontend f = new ReuseFrontend(mediator);
	if (!proof.getJavaModel().isEmpty()) {
	    try{
		diff = cvs.cvsDiff(proof.getJavaModel().getCVSModule(), //XXX: check equality
				   prev.getJavaModel().getModelTag(), 
				   proof.getJavaModel().getModelTag());
		error = f.markup(prev, diff);
	    } catch(CvsException cvse) {
		mgtLogger.error("Diffing models in CVS failed: "+cvse);
	    }
	} else f.markRoot(prev);
	
	if (error != null) JOptionPane.showMessageDialog
			       (mediator.mainFrame(), error, "Oops...",
				JOptionPane.ERROR_MESSAGE);
    }
    

    private Proof[] lookupPrevious(Proof p) {
	List result = new LinkedList();
	Iterator it = envKeyToEnv.values().iterator();
	while (it.hasNext()) {
	    ProofEnvironment env = (ProofEnvironment) it.next();
	    Iterator proofListIt = env.getProofs().iterator();
	    while (proofListIt.hasNext()) {
		ProofList pl = (ProofList) proofListIt.next();
		for (int i=0; i<pl.size(); i++) {
		    if (p != pl.get(i) 
			&& p.mgt().proofSimilarTo(pl.get(i))) {
			result.add(pl.get(i));
		    }
		}
	    }
	}
	return (Proof[]) result.toArray(new Proof[0]);
    }

    static class EnvKey {
        private JavaModel jModel;
	private RuleConfig ruleConfig;

	public EnvKey (JavaModel jModel, RuleConfig ruleConfig) {
	    this.jModel = jModel;
	    this.ruleConfig = ruleConfig;
	}

	public JavaModel getJavaModel() {
	    return jModel;
	}

	public RuleConfig getRuleConfig() {
	    return ruleConfig;
	}

	public boolean equals(Object o) {
	    if (!(o instanceof EnvKey)) return false;
	    EnvKey e = (EnvKey) o;
	    return e.getJavaModel().equals(getJavaModel()) 
		&& e.getRuleConfig().equals(getRuleConfig());
	}

	public int hashCode() {
	    int result=17;
 	    result = 37*result +getJavaModel().hashCode();
 	    result = 37*result +getRuleConfig().hashCode();
	    return result;
	}
    }

}
